#!/usr/bin/env entity

<object name="BugMe">
  
  <object name="todo-renderer">
    
    <renderer name="todo-renderer"
     tag="todo-entry"
     lang="perl">

      <attrib name="__configure"
       description="Bring up attribute dialog."
       value_desc="boolean"
       values="false,true"
       onset="configure_attr_set" />
       
      <perl name="configure_attr_set">
      <![CDATA[
      sub configure_attr_set
      {
        my ($todo_node, $attr, $value) = @_;
	
	my $obj = enode ("object.todo-interface");
	my $xml = $obj->get_xml ();
	
	$todo_node->append_xml ($xml);
      }
      ]]>
      </perl>
    </renderer>
  
  
  <!-- UI template for a single bug report.  When when the 'configure'
       attribute of the 'todo-entry' tag is set to "true", this will be
       rendered in as a child.  We'll fill in the todo-entry-node attribute
       of the object so we can easily delete/update it as needed.  Once
       A change is OK'd by the user, we update the todo-entry-node, 
       call the onchange="" callback, and delete ourselves. -->
  
  <norender>
    <object name="todo-interface">
      <window title="Entity TODO" 
       name="details" visible="true" 
       ondelete="cancel_details"
       height="400"
       width="600"
       position="center">
        <valign expand="true">
	  <halign>
	    <frame title="Description" border="3" expand="true">
	      <entry name="description" text=""/>
	    </frame>
	    <frame title="Status" border="3" expand="true">
	      <dropdown name="status" text="Pending">
	        <string text="Pending"/>
		<string text="Completed"/>
		<string text="Ugh - PITA"/>
	      </dropdown>
	    </frame>
	  </halign>
	  <frame expand="true" title="Notes" border="3">
	    <scrollwin expand="true">
	      <text name="notes" editable="true" expand="true"/>
	    </scrollwin>
	  </frame>
	  <halign>
	    <button label="OK" onclick="store_details" expand="true"/>
	    <button label="Cancel" onclick="cancel_details" expand="true"/>
	  </halign>
	</valign>
      </window>
      <perl name="todo-interface">
      <![CDATA[
        
	sub store_details
	{
	  my $todo = todo_node ();
	  
	  $todo->attrib ("description" => enode ("entry.description")->attrib("text"));
	  $todo->attrib ("status" => enode ("dropdown.status")->attrib("text"));
	  $todo->attrib ("notes" => enode ("text.notes")->get_data ());
	  
	  enode ("object.todo-interface")->delete ();
	  
	  # Let parent know we're done
	  $todo->call ($todo->attrib ("onchange"));
	}
	
	sub cancel_details
	{
	  my $todo = todo_node ();
	  
	  # If we're cancelling a brand new one, we delete
	  # The new record as well as the interface 
	  
	  if (!defined ($todo->attrib ("status"))) {
	    $todo->delete ();
	  } else {
	    enode ("object.todo-interface")->delete ();
	  }
	
	}
	
	sub todo_node
	{
	  my $obj = enode ("object.todo-interface");
	  my $todo = $obj->parent ("todo-entry");
	  return ($todo);
	}

	# Initialize interface settings
	my $todo = todo_node ();
	if ($todo->attrib ("status") ne "") {
	  enode ("entry.description")->attrib("text" => $todo->attrib("description"));
	  enode ("dropdown.status")->attrib ("text" => $todo->attrib("status"));
	  my $data = $todo->attrib("notes");
	  enode ("text.notes")->set_data ($data);
	} else {
	  # Need to init date
	  $todo->attrib ("date" => scalar (localtime()));
	}	
	
      ]]>
      </perl>
      
    </object>
  </norender>
  </object>
  
  
  <!-- Main UI -->
  <window name="main" title="Entity TODO" position="center" width="580"
   height="400" ondelete="entity:exit">
  
    <!-- Node under which to store new entries -->
    <data name="entries"/>
    <clist name="summary" cols="4" width="150"
     selection-sytle="browse" expand="true" onselect="entry_details">
       <cl-title>
	  <string text="Number"/>
	  <string text="Description"/>
	  <string text="Status"/>
	  <string text="Date"/>
       </cl-title>
    </clist>
    
    <halign>
      <button expand="true" onclick="new_entry" label="New Item"/>
      <button expand="true" onclick="save" label="Save"/>
    </halign>
  </window>
  <perl name="main-ui">
  <![CDATA[

    sub new_entry
    {
      my $data = enode ("data.entries");
      my $todo = $data->new_child ("todo-entry");
      
      # Enable config dialog
      $todo->attrib ("onchange" => "refresh_list");
      $todo->attrib ("__configure" => "true");
    }
    
    sub entry_details
    {
      my ($row) = @_;
      
      my $todo = enode ($row->attrib("todo"));
      $todo->attrib ("__configure" => "true");
    }
    
    sub refresh_list
    {
      my $clist = enode ("clist.summary");
      my @list = $clist->children ();
      
      # Delete old entries
      foreach $row (@list) {
        # Don't want to delete the header
	if ($row->attrib("name") ne "header") {
	  $row->delete ();
	}
      }
      
      # Freeze the list so it looks snappier.
      enode ("clist.summary")->attrib ("frozen" => "true"); 
      my $data = enode ("data.entries");
      @list = $data->children ("todo-entry");
      	
      my $todo;
      foreach $todo (@list) {
	  add_to_list ($todo);
      }
      enode ("clist.summary")->attrib ("frozen" => "false"); 
    }
    
    sub add_to_list
    {
      my ($todo) = @_;
      
      # Make sure id is assigned 
      if ($todo->attrib ("bug-number") eq "") {
        assign_number ($todo);
      }
      
      my $description = $todo->attrib ("description");
      my $status = $todo->attrib ("status");
      my $date = $todo->attrib ("date");
      my $number = $todo->attrib ("bug-number");
      
      my $xml = qq!<cl-row todo="$todo">
      		   <string text="$number"/>
      		   <string text="$description"/>
		   <string text="$status"/>
		   <string text="$date"/>
		   </cl-row>!;
      
      enode ("clist.summary")->append_xml ($xml);
    }
    
    sub assign_number
    {
      my ($todo) = @_;
      my $state = enode ("state.main");
      my $bugnum = $state->attrib ("bug-index");
      $todo->attrib ("bug-number" => $bugnum);
      $state->attrib ("bug-index" => ++$bugnum);
    }
    
    sub bugme_db_filename
    {
      my $app_filename = enode ("object.BugMe")->attrib ("__filename");
      my ($path) = $app_filename =~ /(.*)\/.*/;
      my $filename = "$path/bugme.todo";
      
      return ($filename);
    }

    sub save
    {
      my $xml = enode ("data.entries")->get_child_xml ();
      
      my $filename = bugme_db_filename ();
      open (F, ">$filename");
      print F $xml;
      close (F);
    }
    
    sub load
    {
      my $filename = bugme_db_filename ();
      open (F, "$filename") || return;
      my @lines = <F>;
      close (F);
      
      my $xml = join "\n", @lines;
      
      my $newtodo = enode ("data.entries");
      $newtodo->append_xml ($xml);
    }
    
    # Refresh list on init
    load ();
    refresh_list ();

  ]]>
  </perl>
</object>
